// Arctan.java                                      MM 2013

/**
 * Logarithmusfunktion f(x) = arctan( x),
 * D = R, W = ] -PI/2, PI/2[.
 * @author Monika Meiler
 */
public class Arctan extends DifferenzierbareFunktion
{
/* ----------------------------------------------------- */
                                         // service-Methode
/**
 * Berechnen eines Funktionswertes.
 * @param arg Argument
 * @return arctan( arg)
 */
  public double wert( double arg)
  {
// Trivialfall
    if( arg == 0) return 0;

// einfache Newtoniteration
//  return myArctan( arg);
// verbesserte Newtoniteration
    return myArctanBesser( arg);
  }

/**
 * Berechnen eines Funktionswertes als Nullstelle der
 * Umkehrfunktion tan( x) - arg mittels Newton.
 * @param arg Argument
 * @return arctan( arg)
 */
  private double myArctan( double arg)
  {
    Iteration it = new Iteration();
    Tangens tan = new Tangens();
    return it.newton_1( tan, arg, 1e-15, 1000);
  }

/**
 * Verbesserte Funktionswertberechnung fuer |x| > 1.
 * @param arg Argument
 * @return arctan( arg)
 */
  private double myArctanBesser( double arg)
  {
    int fak = 1;
    if( Math.abs( arg) > 1 )
    {
      Wurzel sqrt = new Wurzel();
      sqrt.setWurzel( 2);
      fak = 2;
      arg /= (1 + sqrt.wert( 1 + arg * arg));
    }

    return fak * myArctan( arg);
  }

/**
 * Berechnen einer ersten Ableitung
 * f'(x) = 1 / ( 1 + x^2).
 * @param arg Argument
 * @return f'( arg)
 */
  public double wertErsteAbleitung( double arg)
  {
// Trivialfall
    if( arg == 0) return 0;

// Sonst
    return 1 / ( 1 + arg * arg);
  }

/* ----------------------------------------------------- */
                                        // toString-Methode
/**
 * Darstellen der Arcustangensfunktion.
 * @return Funktion in linearer Schreibweise
 */
  public String toString()
  {
    return "arctan( x)";
  }
}