// Wurzel.java                                      MM 2013
import Tools.IO.*;                              // Eingaben

/**
 * Wurzelfunktion f(x) = k. Wurzel( x), k > 0 aus N,
 * D = W = [ 0, +unendlich[.
 * @author Monika Meiler
 */
public class Wurzel extends DifferenzierbareFunktion
{
/* ----------------------------------------------------- */
                                               // Attribute
/**
 * Wurzelexponent.
 */
  private int exponent = 2;

/* ----------------------------------------------------- */
                                            // set-Methoden
/**
 * Setzt Wurzelexponent.
 * @param k Wurzelexponent
 * @return true, bei erfolgreichem Eintrag
 *         false, k < 1
 */
  public boolean setWurzel( int k)
  {
    if( k < 1) return false;

    exponent = k;
    return true;
  }

/**
 * Wurzelexponenteingabe ueber Konsole.
 * @return true, bei erfolgreichem Eintrag
 *         false, k < 1
 */
  public boolean konsolenEingabe()
  {
// Eingabe des Wurzelexponenten
    int k
    = IOTools.readInteger( " Wurzelexponent k ( k > 0) = ");

// Setzen des Wurzelexponenten
    return setWurzel( k);
  }

/* ----------------------------------------------------- */
                                         // service-Methode
/**
 * Liest Wurzelexponent.
 * @return exponent Wurzelexponent
 */
  public int getExponent()
  {
    return exponent;
  }

/**
 * Berechnen eines Funktionswertes als Nullstelle
 * der Funktion x ^ k - arg.
 * @param arg Argument
 * @return f( arg) falls definiert, NaN sonst
 */
  public double wert( double arg)
  {
// Trivialfaelle
    if( arg < 0)  return Double.NaN;
    if( arg == 0) return 0;
    if( exponent == 1)   return arg;

// Newtoniteration
    return myWurzel( arg);
  }

/**
 * Berechnen eines Funktionswertes als Nullstelle
 * der Funktion x ^ k - arg mittels Newton.
 * @param arg Argument
 * @return f( arg)
 */
  private double myWurzel( double arg)
  {
// Polynom x ^ k - arg
    double[] koeff = new double[ exponent + 1];
    koeff[ exponent] = 1;
    koeff[ 0] = -arg;
    for( int i = 1; i < exponent; i++) koeff[ i] = 0;

    Polynom polynom = new Polynom();
    polynom.setPolynom( koeff);
    // System.out.println( "Polynom " + polynom);

// Newtoniteration
    Iteration it = new Iteration();
    return it.newton( polynom, arg, 1e-15, 1000);
  }

/**
 * Berechnen der ersten Ableitung
 * f'(x) = ( k. Wurzel aus x) / ( x * k).
 * @param arg Argument
 * @return f'( arg)
 */
  public double wertErsteAbleitung( double arg)
  {
// Trivialfall
    if( arg == 0) return Double.NaN;

// Sonst
    return wert( arg) / ( arg * exponent);
  }

/* ----------------------------------------------------- */
                                        // toString-Methode
/**
 * Darstellen der Wurzelfunktion.
 * @return Funktion in linearer Schreibweise
 */
  public String toString()
  {
    return "" + exponent + ". Wurzel( x)";
  }
}